<?php

namespace Simpler\Http\Controllers;

use Simpler\Exceptions\InapplicableCouponException;
use Simpler\Http\Schemas\QuotationSchema;
use Simpler\Models\CartItem;
use Simpler\Models\QuotationRequest;
use Simpler\Services\QuotationService;
use WP_REST_Request;
use WP_REST_Response;

class QuotationController extends Controller
{
    protected $namespace = 'wc/simpler/v1';
    /**
     * Route name.
     *
     * @var string
     */
    protected $route = 'orders/quote';

    /**
     * Route methods.
     *
     * @var string
     */
    protected $method = 'POST';
    /**
     * @var QuotationService
     */
    protected $quotationService;

    public function __construct(QuotationService $quotationService)
    {
        parent::__construct();
        $this->quotationService = $quotationService;
    }

    /**
     * @param  WP_REST_Request  $request
     *
     * @return WP_REST_Response
     */
    public function handle($request)
    {
        $validation = \rest_validate_value_from_schema($body = $request->get_json_params(), QuotationSchema::$schema);

        if (\is_wp_error($validation)) {
            return new WP_REST_Response(json_encode($validation), 422);
        }

        $items = [];
        foreach ($body['items'] as $item) {
            $items[] = CartItem::from_json($item);
        }
        try {
            $quotationRequest = (new QuotationRequest($items))->set_coupon_code($body['coupon'] ?? '');
            $quotations       = $this->quotationService->quote($quotationRequest);
        } catch (InapplicableCouponException $exception) {
            return new WP_REST_Response(['discount_cents' => 0]);
        } catch (\Exception $e) {
            return new WP_REST_Response([
                                            'message' => 'Failed to quote cart',
                                            'error'   => $e->getMessage(),
                                        ], 400);
        }

        $discount = isset($quotations[0]) ? $quotations[0]->get_discount_cents() : 0;

        return new WP_REST_Response(['discount_cents' => $discount]);
    }


    public function get_permission_callback()
    {
        return $this->WCBasicAuth();
    }
}
