<?php

namespace Simpler\Services;

use Exception;
use Simpler\Exceptions\BaseException;
use Simpler\Exceptions\InapplicableCouponException;
use Simpler\Exceptions\IndividuallySoldProductException;
use Simpler\Exceptions\InvalidCouponException;
use Simpler\Exceptions\InvalidProductException;
use Simpler\Exceptions\OutOfStockProductException;
use Simpler\Exceptions\UnpurchasableProductException;
use Simpler\Exceptions\UnshippableCartException;
use Simpler\Models\{CartItem, Money, Quotation, QuotationRequest};
use Simpler\Exceptions\UnshippableLocationException;

class QuotationService
{
    /**
     * @var QuotationRequest
     */
    private $request;
    /**
     * Add product to cart exception
     *
     * @var BaseException
     */
    private $addProductToCartException;

    /**
     *
     * @return Quotation[]
     * @throws Exception
     */
    public function quote(QuotationRequest $request): array
    {
        $this->init();
        $quotations    = [];
        $this->request = $request;

        foreach ($this->request->get_items() as $item) {
            $this->add_item_to_cart($item);
        }

        // Handle shipping
        if ($this->request->get_shipping_address() !== null) {
            foreach ($this->calculate_shipping_rates() as $shippingRate) {
                $quotations[] = (new Quotation())->set_shipping_rate($shippingRate);
            }
        }

        // Handle coupon
        if ($this->request->get_coupon_code()) {
            if (!$this->coupon_exists()) {
                throw new InvalidCouponException('Supplied coupon does not exist');
            }

            if (empty($quotations)) {
                $quotations[] = (new Quotation());
            }

            $couponApplied = WC()->cart->apply_coupon($couponCode = mb_strtolower($this->request->get_coupon_code()));
            if (!$couponApplied) {
                throw new InapplicableCouponException(json_encode(WC()->session->get('wc_notices', [])));
            }

            $discount = Money::to_cents(WC()->cart->get_coupon_discount_amount($couponCode, !wc_prices_include_tax()));
            foreach ($quotations as $quotation) {
                $quotation->set_discount_cents($discount);
            }
        }

        $this->calculate_quotations_total($quotations);

        return $quotations;
    }

    /**
     * @throws BaseException|Exception
     */
    private function add_item_to_cart(CartItem $item)
    {
        $productAdded = WC()->cart->add_to_cart($item->get_product_id(), $item->get_quantity());
        if (is_bool($productAdded) && !$productAdded) {
            throw $this->addProductToCartException
                ?: new InvalidProductException(json_encode(WC()->session->get('wc_notices')));
        }
    }

    /**
     * Determine whether given coupon exists or not.
     *
     * @return bool
     */
    private function coupon_exists(): bool
    {
        return wc_get_coupon_id_by_code($this->request->get_coupon_code()) > 0;
    }

    /**
     * @return \WC_Shipping_Rate[]
     * @throws Exception|UnshippableCartException
     */
    protected function calculate_shipping_rates()
    {
        $customerProperties = [
            'shipping_country'   => $this->request->get_shipping_address()->get_country(),
            'shipping_postcode'  => $this->request->get_shipping_address()->get_postcode(),
            'shipping_city'      => $this->request->get_shipping_address()->get_city(),
            'shipping_address_1' => $this->request->get_shipping_address()->get_address(),
        ];

        if ($this->request->get_shipping_address()->get_state()) {
            $customerProperties['shipping_state'] = $this->request->get_shipping_address()->get_state();
        }

        WC()->customer = new \WC_Customer(0, true);
        WC()->customer->set_props($customerProperties);
        WC()->customer->save();
        WC()->cart->calculate_shipping();
        WC()->cart->calculate_totals();
        if (!WC()->cart->needs_shipping()) {
            throw new UnshippableCartException('no shipping methods available for product and address');
        }

        $packages = WC()->cart->get_shipping_packages();
        if (count($packages) > 1) {
            throw new UnshippableCartException('multiple shipping packages are not supported');
        }

        $shipping = WC()->shipping()->calculate_shipping($packages);

        if (isset($shipping[0]) && isset($shipping[0]['rates']) && !empty($shipping[0]['rates'])) {
            return $shipping[0]['rates'];
        }

        throw new UnshippableLocationException('cart can not be shipped to this address');
    }

    /**
     * Set total cents to quotation.
     *
     * @param  Quotation[]  $quotations
     */
    private function calculate_quotations_total(array $quotations)
    {
        $cartContentTotal = wc_prices_include_tax()
            ? WC()->cart->get_cart_contents_total() + WC()->cart->get_cart_contents_tax()
            : WC()->cart->get_cart_contents_total();

        foreach ($quotations as $quotation) {
            $shippingCost = $quotation->get_shipping_rate() ? (float)$quotation->get_shipping_rate()->get_cost() : 0;
            $total        = absint(Money::to_cents($shippingCost + $cartContentTotal));
            $quotation->set_total_cents($total);
        }
    }

    /**
     * Register WordPress filters
     */
    private function register_filters()
    {
        add_filter(
            'woocommerce_cart_product_cannot_add_another_message',
            [$this, 'individually_sold_product_exception']
        );
        add_filter(
            'woocommerce_cart_product_cannot_be_purchased_message',
            [$this, 'unpurchasable_product_exception']
        );
        add_filter(
            'woocommerce_cart_product_out_of_stock_message',
            [$this, 'out_of_stock_product_exception']
        );
        add_filter(
            'woocommerce_cart_product_not_enough_stock_message',
            [$this, 'out_of_stock_product_exception']
        );
        add_filter(
            'woocommerce_cart_product_not_enough_stock_already_in_cart_message',
            [$this, 'out_of_stock_product_exception']
        );
        // Disable updating cart, in metadata settings (wp_usermeta), of currently authenticated user.
        // This prevents updating an existing cart with the one given in the quotation request.
        add_filter('woocommerce_persistent_cart_enabled', '__return_false');
    }

    /**
     * Register WordPress filters
     */
    private function register_actions()
    {
        // Destroy session, and the DB record that comes with it, that was automatically created during WC_Cart instantiation.
        add_action('shutdown', function () {
            WC()->session->destroy_session();
        });
    }

    private function init()
    {
        $this->register_filters();
        $this->register_actions();
        WC()->cart     = new \WC_Cart();
        // Prevent loading an existing cart, created by the user the API belongs to via the browser, resulting in
        // merging any existing products with the content of the quotation request.
        WC()->session = new \WC_Session_Handler();
    }

    /**
     * Thrown when a product that is sold individually already exists in cart.
     *
     * @param $message
     *
     * @return string
     */
    public function individually_sold_product_exception($message)
    {
        $this->addProductToCartException = new IndividuallySoldProductException($message);

        return $message;
    }

    /**
     * Thrown when a product can not be purchased.
     *
     * @param $message
     *
     * @return string
     */
    public function unpurchasable_product_exception($message)
    {
        $this->addProductToCartException = new UnpurchasableProductException($message);

        return $message;
    }

    /**
     * Thrown when a product:
     * 1. is out of stock
     * 2. doesn't have enough stock
     * 3. doesn't have enough stock accounting for what's already in-cart
     *
     * @param $message
     *
     * @return string
     */
    public function out_of_stock_product_exception($message)
    {
        $this->addProductToCartException = new OutOfStockProductException(wc_clean($message));

        return $message;
    }
}
